library Permanent_Memory_Ex;

// Permanent Memory
// ProfiLab Expert V3.0 or higher
// (www.abacom-online.de)
// Feedback to: Expert.Profi@gmx.de

uses
  SysUtils,
  Windows,
  Classes,
  IniFiles,
  Forms,
  Controls,
  Dlg_MemFile in 'Dlg_MemFile.pas' {MemFileDlg};

{$R *.RES}

const  Inputs = 18;  // number of inputs
       Outputs = 16; // number of outputs

       {DLL INPUTS}
const
                // 0..15 = write data channels
       WR = 16; // Write trigger
       RD = 17; // Read trigger

       {DLL OUTPUTS}
                //0..15 = read data channels

       {USER}
       WR_old  = 50; //
       RD_old  = 51; //


Type TDLLParams = array[0..100] of extended;   // Type of ProfiLab DLL parameters
     PDLLParams = ^TDLLParams;                 // Pointer to ProfiLab DLL parameters

function NumInputs: Byte;
begin
  result:=inputs;
end;

function NumOutputs: Byte;
begin
  result:=outputs;
end;

Function InputName(Channel: Byte): ShortString; // Return name for each component input pin
begin
   case Channel of
   WR: result:='/WR';
   RD: result:='/RD';
   else result:='W'+intToStr(Channel);
   end;
end;

Function OutputName(Channel: Byte): ShortString; // Return name for each component output pin
begin
   result:='R'+intToStr(Channel);
end;

Procedure SimStart(PInput,POutput,PUser: PDLLParams); //called when ProfiLab enters RUN mode
begin
   //not needed
end;

Procedure SimStop(PInput,POutput,PUser: PDLLParams); //called when ProfiLab RUN mode is terminated
begin
   //not needed
end;

Procedure Calculate(PInput,POutput,PUser: PDLLParams); //called regularly from ProfiLab
var R,W, R_old, W_old: Boolean;
    IniFile: TIniFile;

   procedure ReadData; //Read output values from file
   var i: Integer;
   begin
      IniFile:=TIniFile.Create(ExtractFilePath(Application.Exename)+'Data\Permanent_Memory'+intToStr(round(PUser^[99]))+'.ini');
      For i:=0 to 15 do POutput^[i]:=IniFile.ReadFloat('Permanent','Memory'+intToStr(i),0);
      IniFile.Free;
   end;

   procedure WriteData; //Write input values to file
   var i: Integer;
   begin
      IniFile:=TIniFile.Create(ExtractFilePath(Application.Exename)+'Data\Permanent_Memory'+intToStr(round(PUser^[99]))+'.ini');
      For i:=0 to 15 do IniFile.WriteFloat('Permanent','Memory'+intToStr(i),PInput^[i]);
      IniFile.Free;
   end;

begin
   R:=PInput^[RD]>2.5;
   W:=PInput^[WR]>2.5;
   R_old:=PUser^[RD_old]>2.5;
   W_old:=PUser^[WR_old]>2.5;

   if (not R) and R_old then ReadData; //falling edge of RD
   if (not W) and W_old then WriteData; //falling edge of WR

   PUser^[RD_old]:=PInput^[RD];
   PUser^[WR_old]:=PInput^[WR];
end;

Procedure Configure(PUser: PDLLParams);
begin
   MemFileDlg:=TMemFileDlg.Create(nil);
   With MemFileDlg do
   begin
      UpDown1.Position:=round(PUser^[99]);
      if ShowModal = mrOK then
      begin
         PUser^[99]:=UpDown1.Position;
      end;
      Free;
   end;
end;


//export methods for ProfiLab
exports SimStart,
        SimStop,
        NumInputs,
        NumOutputs,
        Calculate,
        InputName,
        OutputName,
        Configure;
begin
end.

